@@ -32,13 +32,13 @@
 "XVkbd*Repeater.initialDelay: 600",
 "XVkbd*Repeater.minimumDelay: 100",
 "XVkbd*Repeater.decay: 500",
-"XVkbd*Font: -*-helvetica-medium-r-*-*-12-*-*-*-*-*-iso8859-*",
-"XVkbd*generalFont: -*-lucidatypewriter-bold-r-*-*-12-*-*-*-*-*-iso8859-*",
-"XVkbd*letterFont: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-*",
-"XVkbd*specialFont: -*-helvetica-medium-r-*-*-12-*-*-*-*-*-iso8859-*",
-"XVkbd*keypadFont: -*-helvetica-medium-r-*-*-10-*-*-*-*-*-iso8859-*",
-"XVkbd*banner.font: -*-lucida-bold-i-*-*-14-*-*-*-*-*-iso8859-*",
-"XVkbd*display_panel*label.font: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-*",
+"XVkbd*Font: -*-helvetica-medium-r-*-*-12-*-*-*-*-*-iso8859-1",
+"XVkbd*generalFont: -*-lucidatypewriter-bold-r-*-*-12-*-*-*-*-*-iso8859-1",
+"XVkbd*letterFont: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-1",
+"XVkbd*specialFont: -*-helvetica-medium-r-*-*-12-*-*-*-*-*-iso8859-1",
+"XVkbd*keypadFont: -*-helvetica-medium-r-*-*-10-*-*-*-*-*-iso8859-1",
+"XVkbd*banner.font: -*-lucida-bold-i-*-*-14-*-*-*-*-*-iso8859-1",
+"XVkbd*display_panel*label.font: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-1",
 "XVkbd*beNiceToColormap: false",
 "XVkbd*topShadowContrast: 40",
 "XVkbd*bottomShadowContrast: 80",
@@ -89,6 +89,7 @@
 "XVkbd*Delete.width: 45",
 "XVkbd*Return.width: 60",
 "XVkbd*row5.Focus.width: 35",
+"xvkbd*Mode_switch.width: 45",
 "XVkbd*keypad.vertDistance: 5",
 "XVkbd*keypad.horizDistance: 10",
 "XVkbd*keypad*Command.width: 35",
@@ -97,7 +98,8 @@
 "XVkbd*keypad*Repeater.justify: left",
 "XVkbd*keypad*pad3,3.height: 60",
 "XVkbd*keypad*pad4,0.width: 70",
-"XVkbd*balloon_panel*background: LightYellow1",
+"XVkbd.balloonBackground: LightYellow1",
+"XVkbd.launchBalloonBackground: LightCyan1",
 "XVkbd*balloon_panel.Label.width: 150",
 "XVkbd*balloon_panel.Label.resize: false",
 "XVkbd*balloon_panel.Label.justify: left",
@@ -112,6 +114,7 @@
 "XVkbd*menu.man.label: Manual...",
 "XVkbd*menu.keypad.label: Keypad...",
 "XVkbd*menu.sun_fkey.label: Sun Function Keys...",
+"XVkbd*menu.deadkey.label: Dead Keys...",
 "XVkbd*menu.completion.label: Word Completion...",
 "XVkbd*menu.select_layout.label: Change Keyboard Layout...",
 "XVkbd*menu.edit_fkey.label: Edit Function Keys...",
@@ -140,17 +143,12 @@
 "XVkbd.display_panel*Command.internalWidth: 4",
 "XVkbd.display_panel*Label.width: 0",
 "XVkbd.display_panel*Label.resizable: true",
-"XVkbd.customizations: default,german,swissgerman,french,latin1,jisx6004/ja",
 "XVkbd*layout_panel.title: xvkbd - layout",
 "XVkbd*layout_panel*Command.width: 120",
 "XVkbd*layout_panel*Command.height: 0",
 "XVkbd*layout_panel*vSpace: 2",
-"XVkbd*layout_panel*default.label: Default",
-"XVkbd*layout_panel*german.label: German",
-"XVkbd*layout_panel*swissgerman.label: Swiss/German",
-"XVkbd*layout_panel*french.label: French",
-"XVkbd*layout_panel*latin1.label: Latin-1",
-"XVkbd*layout_panel*jisx6004.label: Japanese",
+"XVkbd*layout_panel.box.width: 260",
+"XVkbd*layout_panel.box.orientation: vertical",
 "XVkbd*completion_panel.title: xvkbd - word completion",
 "XVkbd*completion_panel*Label.justify: left",
 "XVkbd*completion_panel*entry.width: 200",
@@ -159,8 +157,8 @@
 "XVkbd*completion_panel*view.forceBars: true",
 "XVkbd*completion_panel*view.list.width: 400",
 "XVkbd*completion_panel*view.list.height: 150",
-"XVkbd*completion_panel*label.font: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-*",
-"XVkbd*completion_panel*list.font: -*-lucida-medium-r-*-*-14-*-*-*-*-*-iso8859-*",
+"XVkbd*completion_panel*label.font: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-1",
+"XVkbd*completion_panel*list.font: -*-lucida-medium-r-*-*-14-*-*-*-*-*-iso8859-1",
 "XVkbd*completion_panel*label.label: Completion:",
 "XVkbd*completion_panel*bottom: chainTop",
 "XVkbd*completion_panel*label.right: chainLeft",
@@ -174,7 +172,7 @@
 "XVkbd*edit_fkey_panel*form2*vertDistance: 6",
 "XVkbd*edit_fkey_panel*width: 0",
 "XVkbd*edit_fkey_panel*height: 0",
-"XVkbd*edit_fkey_panel*internalWidth: 2",
+"XVkbd*edit_fkey_panel*internalWidth: 4",
 "XVkbd*edit_fkey_panel*MenuButton.shadowWidth: 2",
 "XVkbd*edit_fkey_panel*form2.borderWidth: 1",
 "XVkbd*edit_fkey_panel*form2.?.left: chainLeft",
@@ -183,17 +181,64 @@
 "XVkbd*edit_fkey_panel*Command.left: chainLeft",
 "XVkbd*edit_fkey_panel*Command.right: chainLeft",
 "XVkbd*edit_fkey_panel*fkey_menu.fromHoriz: fkey_label",
-"XVkbd*edit_fkey_panel*fkey_value_label.fromVert: fkey_label",
+"XVkbd*edit_fkey_panel*fkey_value_menu.fromVert: fkey_label",
 "XVkbd*edit_fkey_panel*fkey_value.fromVert: fkey_label",
-"XVkbd*edit_fkey_panel*fkey_value.fromHoriz: fkey_value_label",
+"XVkbd*edit_fkey_panel*fkey_value_sep.fromVert: fkey_label",
+"XVkbd*edit_fkey_panel*fkey_value_sep.fromHoriz: fkey_value_menu",
+"XVkbd*edit_fkey_panel*fkey_value.fromHoriz: fkey_value_sep",
 "XVkbd*edit_fkey_panel*fkey_value.width: 200",
 "XVkbd*edit_fkey_panel*fkey_value.height: 18",
 "XVkbd*edit_fkey_panel*save_button.fromVert: form2",
 "XVkbd*edit_fkey_panel*close_button.fromVert: form2",
 "XVkbd*edit_fkey_panel*close_button.fromHoriz: save_button",
-"XVkbd*edit_fkey_panel*fkey_menu.label: F99",
+"XVkbd*edit_fkey_panel*justify: left",
+"XVkbd*edit_fkey_panel*fkey_menu.label: Shift-F999",
 "XVkbd*edit_fkey_panel*fkey_label.label: Function Key:",
-"XVkbd*edit_fkey_panel*fkey_value_label.label: Assigned String:",
+"XVkbd*edit_fkey_panel*fkey_value_menu.label: Assigned Command",
+"XVkbd*edit_fkey_panel*fkey_value_menu*string.label: Assigned String",
+"XVkbd*edit_fkey_panel*fkey_value_menu*command.label: Assigned Command",
+"XVkbd*edit_fkey_panel*fkey_value_sep.label: :",
+"XVkbd*edit_fkey_panel*fkey_value_sep.horizDistance: 0",
 "XVkbd*edit_fkey_panel*save_button.label: Save",
 "XVkbd*edit_fkey_panel*close_button.label: Close",
-"XVkbd*edit_fkey_panel*Label.font: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-*",
+"XVkbd*edit_fkey_panel*Label.font: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-1",
+"XVkbd*edit_fkey_panel*fkey_value_menu.font: -*-helvetica-bold-r-*-*-12-*-*-*-*-*-iso8859-1",
+"XVkbd.deadkeys: dead_diaeresis,dead_acute,dead_grave,\
+  dead_circumflex,dead_macron,dead_abovering,dead_doubleacute,\
+  dead_tilde,dead_abovedot",
+"XVkbd*deadkey_panel.title: xvkbd - dead keys",
+"XVkbd*deadkey_panel*font: -*-helvetica-medium-r-*-*-24-*-*-*-*-*-iso8859-1",
+"XVkbd*deadkey_panel*Repeater.width: 30",
+"XVkbd*deadkey_panel*Repeater.height: 36",
+"XVkbd*deadkey_panel*dead_diaeresis.label: \\250",
+"XVkbd*deadkey_panel*dead_acute.label: \\264",
+"XVkbd*deadkey_panel*dead_grave.label: `",
+"XVkbd*deadkey_panel*dead_circumflex.label: ^",
+"XVkbd*deadkey_panel*dead_macron.label: \\257",
+"XVkbd*deadkey_panel*dead_abovering.label: \\260",
+"XVkbd*deadkey_panel*dead_doubleacute.label: \\264\\264",
+"XVkbd*deadkey_panel*dead_tilde.label: ~",
+"XVkbd*deadkey_panel*dead_abovedot.label: \\267",
+"XVkbd.customizations: default,belgian,danish,french,french2,\
+  german,greek,hebrew,icelandic,italian,jisx6004/ja,norwegian,\
+  portuguese,slovene,spanish,swedish,swissgerman,turkish,uk,latin1",
+"XVkbd*layout_panel*default.label: Default",
+"XVkbd*layout_panel*belgian.label: Belgian",
+"XVkbd*layout_panel*danish.label: Danish",
+"XVkbd*layout_panel*french.label: French",
+"XVkbd*layout_panel*french2.label: French (2)",
+"XVkbd*layout_panel*german.label: German",
+"XVkbd*layout_panel*greek.label: Greek",
+"XVkbd*layout_panel*hebrew.label: Hebrew",
+"XVkbd*layout_panel*icelandic.label: Icelandic",
+"XVkbd*layout_panel*italian.label: Italian",
+"XVkbd*layout_panel*jisx6004.label: Japanese",
+"XVkbd*layout_panel*norwegian.label: Norwegian",
+"XVkbd*layout_panel*portuguese.label: Portuguese",
+"XVkbd*layout_panel*slovene.label: Slovene",
+"XVkbd*layout_panel*spanish.label: Spanish",
+"XVkbd*layout_panel*swedish.label: Swedish",
+"XVkbd*layout_panel*swissgerman.label: Swiss/German",
+"XVkbd*layout_panel*turkish.label: Turkish",
+"XVkbd*layout_panel*uk.label: United Kingdom",
+"XVkbd*layout_panel*latin1.label: Latin-1",
