@@ -59,6 +59,8 @@ struct appres_struct {
   Pixel highlight_foreground;
   Pixel focus_background;
   Pixel remote_focus_background;
+  Pixel balloon_background;
+  Pixel launch_balloon_background;
 
   String keys_normal;
   String keys_shift;
@@ -69,11 +71,15 @@ struct appres_struct {
   String shift_key_labels;
   String altgr_key_labels;
   String shift_altgr_key_labels;
+  String deadkeys;
 
   String key_file;
   String dict_file;
   String customizations;
   int editable_function_keys;
+
+  float max_width_ratio;
+  float max_height_ratio;
 } appres;
 
 
