@@ -1,6 +1,6 @@
 /*
  * xvkbd - Virtual Keyboard for X Window System
- * (Version 2.2, 2002-03-16)
+ * (Version 2.4, 2002-10-02)
  *
  * Copyright (C) 2000-2002 by Tom Sato <VEF00200@nifty.ne.jp>
  * http://member.nifty.ne.jp/tsato/
@@ -52,7 +52,7 @@
 #endif
 
 #include "resources.h"
-#define PROGRAM_NAME_WITH_VERSION "xvkbd (v2.2)"
+#define PROGRAM_NAME_WITH_VERSION "xvkbd (v2.4)"
 
 /*
  * Default keyboard layout is hardcoded here.
@@ -96,18 +96,18 @@ static char *normal_key_labels[NUM_KEY_ROWS][NUM_KEY_COLS] = {
   { "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12", "Backspace" },
   { "Esc", "1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "-", "=", "\\", "`" },
   { "Tab", "q", "w", "e", "r", "t", "y", "u", "i", "o", "p", "[", "]", "Del" },
-  { "Control", "a", "s", "d", "f", "g", "h", "j", "k", "l", ";", "'", "Return" },
-  { "Shift", "z", "x", "c", "v", "b", "n", "m", ",", ".", "/", "Com\npose", "Shift" },
-  { "MainMenu", "Caps\nLock", "Alt", "Meta", "", "Meta", "Alt",
+  { "Ctrl", "a", "s", "d", "f", "g", "h", "j", "k", "l", ";", "'", "Return" },
+  { "Shift", "z", "x", "c", "v", "b", "n", "m", ",", ".", "/", "Comp", "Shift" },
+  { "MainMenu", "Caps", "Alt", "Meta", "", "Meta", "Alt",
     "left", "right", "up", "down", "Focus" },
 };
 static char *shift_key_labels[NUM_KEY_ROWS][NUM_KEY_COLS] = {
   { "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "F10", "F11", "F12", "Backspace" },
   { "Esc", "!", "@", "#", "$", "%", "^", "&", "*", "(", ")", "_", "+", "|", "~" },
   { "Tab", "Q", "W", "E", "R", "T", "Y", "U", "I", "O", "P", "{", "}", "Del" },
-  { "Control", "A", "S", "D", "F", "G", "H", "J", "K", "L", ":", "\"", "Return" },
-  { "Shift", "Z", "X", "C", "V", "B", "N", "M", "<", ">", "?", "Com\npose", "Shift" },
-  { "MainMenu", "Caps\nLock", "Alt", "Meta", "", "Meta", "Alt",
+  { "Ctrl", "A", "S", "D", "F", "G", "H", "J", "K", "L", ":", "\"", "Return" },
+  { "Shift", "Z", "X", "C", "V", "B", "N", "M", "<", ">", "?", "Comp", "Shift" },
+  { "MainMenu", "Caps", "Alt", "Meta", "", "Meta", "Alt",
     "left", "right", "up", "down", "Focus" },
 };
 static char *altgr_key_labels[NUM_KEY_ROWS][NUM_KEY_COLS] = { { NULL } };
@@ -195,6 +195,14 @@ static unsigned char check_bits[] = {
   0x8c, 0x07, 0xde, 0x03, 0xfe, 0x03, 0xfc, 0x01, 0xf8, 0x00, 0xf0, 0x00,
   0x70, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00};
 
+#define back_width 18
+#define back_height 13
+static unsigned char back_bits[] = {
+   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x00, 0x00,
+   0x78, 0x00, 0x00, 0xfe, 0xff, 0x03, 0xff, 0xff, 0x03, 0xfe, 0xff, 0x03,
+   0x78, 0x00, 0x00, 0x60, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
+   0x00, 0x00, 0x00};
+
 /*
  * Resources and options
  */
@@ -282,6 +290,11 @@ static XtResource application_resources[] = {
      Offset(focus_background), XtRString, "gray" },
   { "remoteFocusBackground", XtCBackground, XtRPixel, sizeof(Pixel),
      Offset(remote_focus_background), XtRString, "cyan" },
+  { "balloonBackground", XtCBackground, XtRPixel, sizeof(Pixel),
+     Offset(balloon_background), XtRString, "LightYellow1" },
+  { "launchBalloonBackground", XtCBackground, XtRPixel, sizeof(Pixel),
+     Offset(launch_balloon_background), XtRString, "SkyBlue1" },
+
   { "normalkeys", "NormalKeys", XtRString, sizeof(char *),
     Offset(keys_normal), XtRImmediate, "" },
   { "shiftkeys", "ShiftKeys", XtRString, sizeof(char *),
@@ -300,6 +313,8 @@ static XtResource application_resources[] = {
     Offset(altgr_key_labels), XtRImmediate, "" },
   { "shiftaltgrkeylabels", "ShiftAltgrKeyLabels", XtRString, sizeof(char *),
     Offset(shift_altgr_key_labels), XtRImmediate, "" },
+  { "deadkeys", "DeadKeys", XtRString, sizeof(char *),
+    Offset(deadkeys), XtRImmediate, "" },
 
   { "keyFile", "KeyFile", XtRString, sizeof(char *),
     Offset(key_file), XtRImmediate, ".xvkbd" },
@@ -309,6 +324,11 @@ static XtResource application_resources[] = {
     Offset(customizations), XtRImmediate, "default" },
   { "editableFunctionKeys", "EditableFunctionKeys", XtRInt, sizeof(int),
      Offset(editable_function_keys), XtRImmediate, (XtPointer)12 },
+
+  { "maxWidthRatio", "MaxRatio", XtRFloat, sizeof(float),
+     Offset(max_width_ratio), XtRString, "0.9" },
+  { "maxHeightRatio", "MaxRatio", XtRFloat, sizeof(float),
+     Offset(max_height_ratio), XtRString, "0.5" },
 };
 #undef Offset
 
@@ -355,7 +375,7 @@ static char **argv1;
 static XtAppContext app_con;
 static Widget toplevel;
 static Widget key_widgets[NUM_KEY_ROWS][NUM_KEY_COLS];
-static Widget main_menu;
+static Widget main_menu = None;
 
 static Dimension toplevel_height = 1000;
 
@@ -387,6 +407,7 @@ static void SendString(const unsigned char *str);
 static void MakeKeyboard(Boolean remake);
 static void MakeKeypad(Widget form, Widget from_vert, Widget from_horiz);
 static void MakeSunFunctionKey(Widget form, Widget from_vert, Widget from_horiz);
+static void MakeDeadkeyPanel(Widget form);
 static void RefreshMainMenu(void);
 static void PopupFunctionKeyEditor(void);
 
@@ -737,7 +758,8 @@ static int AddKeysym(KeySym keysym, Boolean top)
   for (pos = max_pos; 0 <= pos; pos--) {
     for (keycode = max_keycode; min_keycode <= keycode; keycode--) {
       inx = (keycode - min_keycode) * keysym_per_keycode;
-      if (keysym_table[inx] <= XK_F35) {  /* if the key is not modifier key */
+      if (keysym_table[inx] < 0xFF00) {
+	/* if the key is not modifier key or other special key */
 	if (keysym_table[inx + pos] == NoSymbol) {
 	  if (appres.debug)
 	    fprintf(stderr, "Adding keysym \"%s\" at keycode %d position %d/%d\n",
@@ -1502,8 +1524,9 @@ static void PopupLayoutPanel(void)
     }
     XtRealizeWidget(layout_panel);
     XSetWMProtocols(dpy, XtWindow(layout_panel), &wm_delete_window, 1);
+
+    XtFree(customizations);
   }
-  XtFree(customizations);
 
   XtPopup(layout_panel, XtGrabNone);
 }
@@ -1514,6 +1537,7 @@ static void PopupLayoutPanel(void)
 static Widget about_panel = None;
 static Widget keypad_panel = None;
 static Widget sun_fkey_panel = None;
+static Widget deadkey_panel = None;
 static Widget display_panel = None;
 static Widget display_status = None;
 
@@ -1592,6 +1616,16 @@ static void MenuSelected(Widget w, char *key)
       XSetWMProtocols(dpy, XtWindow(sun_fkey_panel), &wm_delete_window, 1);
     }
     XtPopup(sun_fkey_panel, XtGrabNone);
+  } else if (strcmp(key, "deadkey") == 0) {
+    if (deadkey_panel == None) {
+      deadkey_panel = XtVaCreatePopupShell("deadkey_panel", transientShellWidgetClass,
+					  toplevel, NULL);
+      form = XtVaCreateManagedWidget("form", formWidgetClass, deadkey_panel, NULL);
+      MakeDeadkeyPanel(form);
+      XtRealizeWidget(deadkey_panel);
+      XSetWMProtocols(dpy, XtWindow(deadkey_panel), &wm_delete_window, 1);
+    }
+    XtPopup(deadkey_panel, XtGrabNone);
   } else if (strcmp(key, "completion") == 0) {
     PopupCompletionPanel();
   } else if (strcmp(key, "select_layout") == 0) {
@@ -1654,9 +1688,6 @@ static void ClosePopupPanel(Widget w)
   if (w == keypad_panel) {
     XtDestroyWidget(w);
     keypad_panel = None;
-  } else if (w == sun_fkey_panel) {
-    XtDestroyWidget(w);
-    sun_fkey_panel = None;
   } else {
     XtPopdown(w);
   }
@@ -1665,7 +1696,7 @@ static void ClosePopupPanel(Widget w)
 /*
  * This will be called when user pressed a key on the screen.
  */
-static const char *FindFunctionKeyValue(const char *key);
+static const char *FindFunctionKeyValue(const char *key, Boolean shiftable);
 static void ShowBalloon(Widget w, XEvent *event, String *pars, Cardinal *n_pars);
 
 static void KeyPressed(Widget w, char *key, char *data)
@@ -1680,9 +1711,17 @@ static void KeyPressed(Widget w, char *key, char *data)
 
   if (appres.debug) fprintf(stderr, "KeyPressed: key=%s, widget=%lx\n", key, (long)w);
 
-  value = FindFunctionKeyValue(key);
+  value = FindFunctionKeyValue(key, TRUE);
   if (value != NULL) {
-    SendString(value);
+    if (appres.debug) fprintf(stderr, "Assigned string: %s\n", value);
+    if (value[0] == '!') {
+      if (appres.debug) fprintf(stderr, "Launching: %s\n", value + 1);
+      system(value + 1);
+    } else {
+      if (value[0] == '\\') value = value + 1;
+      if (appres.debug) fprintf(stderr, "Sending: %s\n", value);
+      SendString(value);
+    }
     ShowBalloon(w, NULL, NULL, NULL);
     return;
   }
@@ -1891,6 +1930,7 @@ static Widget MakeKey(Widget parent, const char *name, const char *label, Pixel
   static Pixmap down_pixmap = None;
   static Pixmap left_pixmap = None;
   static Pixmap right_pixmap = None;
+  static Pixmap back_pixmap = None;
   Widget w;
   Window scr = RootWindow(dpy, DefaultScreen(dpy));
   char str[50];
@@ -1943,6 +1983,11 @@ static Widget MakeKey(Widget parent, const char *name, const char *label, Pixel
         right_pixmap = XCreateBitmapFromData(dpy, scr,
                                 (char *)right_bits, right_width, right_height);
       XtVaSetValues(w, XtNbitmap, right_pixmap, NULL);
+    } else if (strcmp(label, "back") == 0) {
+      if (back_pixmap == None)
+        back_pixmap = XCreateBitmapFromData(dpy, scr,
+                                (char *)back_bits, back_width, back_height);
+      XtVaSetValues(w, XtNbitmap, back_pixmap, NULL);
     }
   }
 
@@ -2032,6 +2077,27 @@ static void MakeSunFunctionKey(Widget form, Widget from_vert, Widget from_horiz)
   }
 }
 
+static void MakeDeadkeyPanel(Widget form)
+{
+  int i;
+  Widget deadkey_box, left, key;
+  char *deadkeys, *cp, *cp2;
+
+  deadkeys = XtNewString(appres.deadkeys);
+
+  deadkey_box = XtVaCreateManagedWidget("deadkey", formWidgetClass, form, NULL);
+  left = None;
+  cp = strtok(deadkeys, " \t,");
+  while (cp != NULL) {
+    cp2 = XtNewString(cp);
+    key = MakeKey(deadkey_box, cp2, NULL, appres.general_background);
+    if (left != None) XtVaSetValues(key, XtNfromHoriz, left, NULL);
+    left = key;
+    cp = strtok(NULL, " \t,");
+  }
+  XtFree(deadkeys);
+}
+
 static void RefreshMainMenu(void)
 {
   static Pixmap check_pixmap = None;
@@ -2060,7 +2126,7 @@ static void RefreshMainMenu(void)
 static void MakeKeyboard(Boolean remake)
 {
   static char *main_menu_items[] = {
-    "about", "man", "keypad", "sun_fkey", "completion", "",
+    "about", "man", "keypad", "sun_fkey", "deadkey", "completion", "",
     "select_layout",
     "edit_fkey",
     "show_keypad",
@@ -2173,6 +2239,27 @@ static void MakeKeyboard(Boolean remake)
   }
 
   XtRealizeWidget(toplevel);
+
+  if (!remake && strlen(appres.geometry) == 0) {
+    Window root;
+    int x1, y1;
+    unsigned int wd, ht, bd, dp;
+    int max_wd, max_ht;
+
+    XGetGeometry(dpy, XtWindow(toplevel), &root, &x1, &y1, &wd, &ht, &bd, &dp);
+    max_wd = XtScreen(toplevel)->width * appres.max_width_ratio;
+    max_ht = XtScreen(toplevel)->height * appres.max_height_ratio;
+    if (appres.debug)
+      fprintf(stderr, "window size: %dx%d, max size: %dx%d\n", wd, ht, max_wd, max_ht);
+    if (max_wd < wd || max_ht < ht) {
+      if (max_wd < wd) wd = max_wd;
+      if (max_ht < ht) ht = max_ht;
+      if (appres.debug)
+	fprintf(stderr, "setting window size: %dx%d\n", wd, ht);
+      XResizeWindow(dpy, XtWindow(toplevel), wd, ht);
+    }
+  }
+
   if (!appres.debug && key_box[first_row] != None) {
     if (appres.keypad) {
       XtVaGetValues(key_box[1], XtNwidth, &max_wd, NULL);
@@ -2188,13 +2275,15 @@ static void MakeKeyboard(Boolean remake)
     }
   }
   if (0 < strlen(appres.geometry)) {
+    if (appres.debug)
+      fprintf(stderr, "setting window geometry: %s\n", appres.geometry);
     XtVaSetValues(toplevel, XtNgeometry, appres.geometry, NULL);
     XtUnrealizeWidget(toplevel);
     XtRealizeWidget(toplevel);
   }
 
   ReadKeymap();
-  RefreshMainMenu();
+  if (main_menu != None) RefreshMainMenu();
   RefreshShiftState(FALSE);
 
   XtMapWidget(toplevel);
@@ -2293,17 +2382,42 @@ static void ReadFuncionKeys(void)
  */
 static Widget edit_fkey_panel = None;
 static Widget fkey_menu_button = None;
+static Widget fkey_value_menu_button = None;
 static Widget fkey_value_entry = None;
 static char fkey_value[100] = "";
-static char *cur_fkey = "";
+static char cur_fkey[20] = "";
+static char *cur_fkey_value_mode = "";
+
+static void FKeyValueMenuSelected(Widget w, char *key)
+{
+  char *key1, *cp;
+
+  if (key[0] == 'c') {
+    cur_fkey_value_mode = "command";
+    key1 = "*command";
+  } else {
+    cur_fkey_value_mode = "string";
+    key1 = "*string";
+  }
+  XtVaGetValues(XtNameToWidget(fkey_value_menu_button, key1), XtNlabel, &cp, NULL);
+  XtVaSetValues(fkey_value_menu_button, XtNlabel, cp, NULL);
+}
 
 static void FKeyMenuSelected(Widget w, char *key)
 {
   struct fkey_struct *sp, *sp2;
   int len;
-  const char *value;
+  const char *value, *prefix;
+  char key2[20];
 
-  if (cur_fkey != key) {
+  if (key == NULL)
+    strcpy(key2, "");
+  else if (strncmp(key, "Shift-", strlen("Shift-")) == 0)
+    sprintf(key2, "s:%s", &key[strlen("Shift-")]);
+  else
+    strcpy(key2, key);
+
+  if (strcmp(cur_fkey, key2) != 0) {
     if (strlen(cur_fkey) != 0) {
       len = strlen(cur_fkey);
       sp2 = NULL;
@@ -2320,10 +2434,11 @@ static void FKeyMenuSelected(Widget w, char *key)
 	  sp->next = NULL;
 	  sp->value = NULL;
 	}
-	if (sp->value == NULL || strcmp(&(sp->value[len + 1]), fkey_value) != 0) {
-	  sp->value = realloc(sp->value, len + strlen(fkey_value) + 2);
-	  sprintf(sp->value, "%s %s", cur_fkey, fkey_value);
-	}
+	sp->value = realloc(sp->value, len + strlen(fkey_value) + 5);
+	prefix = "";
+	if (cur_fkey_value_mode[0] == 'c') prefix = "!";
+	else if (fkey_value[0] == '!') prefix = "\\";
+	sprintf(sp->value, "%s %s%s", cur_fkey, prefix, fkey_value);
       } else {  /* empty string - remove the entry for the function key */
 	if (sp != NULL) {
 	  if (sp2 != NULL) sp2->next = sp->next;
@@ -2337,12 +2452,16 @@ static void FKeyMenuSelected(Widget w, char *key)
     if (key != NULL) {
       XtVaSetValues(fkey_menu_button, XtNlabel, key, NULL);
   
-      value = FindFunctionKeyValue(key);
+      value = FindFunctionKeyValue(key2, FALSE);
       if (value == NULL) value = "";
+
+      FKeyValueMenuSelected(None, (value[0] == '!') ? "command" : "string");
+
+      if (value[0] == '!' || value[0] == '\\') value = value + 1;
       strncpy(fkey_value, value, sizeof(fkey_value) - 1);
       XtVaSetValues(fkey_value_entry, XtNstring, fkey_value, NULL);
 
-      cur_fkey = key;
+      strcpy(cur_fkey, key2);
     }
   }
 }
@@ -2376,9 +2495,9 @@ static void SaveFunctionKey(void)
 static void PopupFunctionKeyEditor(void)
 {
   Widget form, form2, menu, menu_entry, button;
-  char label[10];
+  char label[20];
   char *key;
-  int i;
+  int i, j;
 
   if (edit_fkey_panel == None) {
     edit_fkey_panel = XtVaCreatePopupShell("edit_fkey_panel", transientShellWidgetClass,
@@ -2390,14 +2509,31 @@ static void PopupFunctionKeyEditor(void)
     fkey_menu_button = XtVaCreateManagedWidget("fkey_menu", menuButtonWidgetClass,
 					       form2, NULL);
     menu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, fkey_menu_button, NULL);
-    for (i = 1; i <= appres.editable_function_keys; i++) {
-      sprintf(label, "F%d", i);
-      key = XtNewString(label);
-      menu_entry = XtVaCreateManagedWidget(key, smeBSBObjectClass, menu, NULL);
-      XtAddCallback(menu_entry, XtNcallback, (XtCallbackProc)FKeyMenuSelected,
-		    (XtPointer)key);
+    for (j = 0; j <= 1; j++) {
+      for (i = 1; i <= appres.editable_function_keys; i++) {
+	if (j == 0)
+	  sprintf(label, "F%d", i);
+	else 
+	  sprintf(label, "Shift-F%d", i);
+	key = XtNewString(label);
+	menu_entry = XtVaCreateManagedWidget(key, smeBSBObjectClass, menu, NULL);
+	XtAddCallback(menu_entry, XtNcallback, (XtCallbackProc)FKeyMenuSelected,
+		      (XtPointer)key);
+      }
     }
-    XtVaCreateManagedWidget("fkey_value_label", labelWidgetClass, form2, NULL);
+
+    fkey_value_menu_button = XtVaCreateManagedWidget("fkey_value_menu", menuButtonWidgetClass,
+						     form2, NULL);
+    menu = XtVaCreatePopupShell("menu", simpleMenuWidgetClass, fkey_value_menu_button, NULL);
+    menu_entry = XtVaCreateManagedWidget("string", smeBSBObjectClass, menu, NULL);
+    XtAddCallback(menu_entry, XtNcallback, (XtCallbackProc)FKeyValueMenuSelected,
+		  (XtPointer)"string");
+    menu_entry = XtVaCreateManagedWidget("command", smeBSBObjectClass, menu, NULL);
+    XtAddCallback(menu_entry, XtNcallback, (XtCallbackProc)FKeyValueMenuSelected,
+		  (XtPointer)"command");
+
+    XtVaCreateManagedWidget("fkey_value_sep", labelWidgetClass, form2, NULL);
+
     fkey_value_entry = XtVaCreateManagedWidget("fkey_value", asciiTextWidgetClass, form2,
 					       XtNuseStringInPlace, True,
 					       XtNeditType, XawtextEdit,
@@ -2426,7 +2562,7 @@ static void PopupFunctionKeyEditor(void)
  * If text is assigned to the specified function key,
  * return the text.  Otherwise, return NULL.
  */
-static const char *FindFunctionKeyValue(const char *key)
+static const char *FindFunctionKeyValue(const char *key, Boolean shiftable)
 {
   char label[50];
   char prefix;
@@ -2434,10 +2570,12 @@ static const char *FindFunctionKeyValue(const char *key)
   int len;
 
   prefix = '\0';
-  if (shift_state & meta_mask) prefix = 'm';
-  else if (shift_state & alt_mask) prefix = 'a';
-  else if (shift_state & ControlMask) prefix = 'c';
-  else if (shift_state & ShiftMask) prefix = 's';
+  if (shiftable) {
+    if (shift_state & meta_mask) prefix = 'm';
+    else if (shift_state & alt_mask) prefix = 'a';
+    else if (shift_state & ControlMask) prefix = 'c';
+    else if (shift_state & ShiftMask) prefix = 's';
+  }
   if (prefix == '\0') sprintf(label, "%s", key);
   else sprintf(label, "%c:%s", prefix, key);
   len = strlen(label);
@@ -2464,7 +2602,7 @@ static void ShowBalloon(Widget w, XEvent *event, String *pars, Cardinal *n_pars)
   const char *value;
 
   if (strcmp(XtName(w), "MainMenu") == 0) value = "Main menu";
-  else value = FindFunctionKeyValue(XtName(w));
+  else value = FindFunctionKeyValue(XtName(w), TRUE);
   if (value == NULL) return;
 
   if (balloon_panel == None) {
@@ -2475,7 +2613,14 @@ static void ShowBalloon(Widget w, XEvent *event, String *pars, Cardinal *n_pars)
   XtVaGetValues(w, XtNheight, &ht, NULL);
   XtTranslateCoords(w, 6, ht + 2, &x, &y);
   XtVaSetValues(balloon_panel, XtNx, x, XtNy, y, NULL);
-  XtVaSetValues(message, XtNlabel, value, NULL);
+  if (value[0] == '!') {
+    XtVaSetValues(message, XtNlabel, value + 1,
+		  XtNbackground, appres.launch_balloon_background, NULL);
+  } else {
+    if (value[0] == '\\') value = value + 1;
+    XtVaSetValues(message, XtNlabel, value,
+		  XtNbackground, appres.balloon_background, NULL);
+  }
   XtPopup(balloon_panel, XtGrabNone);
 
   balloon_panel_open = TRUE;
@@ -2625,10 +2770,12 @@ int main(int argc, char *argv[])
       if (child != None) focused_subwindow = child;
     }
 
-    if (strlen(appres.dict_file) == 0)
-      XtSetSensitive(XtNameToWidget(main_menu, "*completion"), FALSE);
-    if (strlen(appres.customizations) == 0)
-      XtSetSensitive(XtNameToWidget(main_menu, "*select_layout"), FALSE);
+    if (main_menu != None) {
+      if (strlen(appres.dict_file) == 0)
+	XtSetSensitive(XtNameToWidget(main_menu, "*completion"), FALSE);
+      if (strlen(appres.customizations) == 0)
+	XtSetSensitive(XtNameToWidget(main_menu, "*select_layout"), FALSE);
+    }
 
 #ifdef USE_XTEST
     if (!XQueryExtension(dpy, "XTEST", &op, &ev, &err)) {
@@ -2639,8 +2786,10 @@ int main(int argc, char *argv[])
 		PROGRAM_NAME);
 	appres.xtest = FALSE;
       }
-      XtSetSensitive(XtNameToWidget(main_menu, "*use_xtest"), FALSE);
-      RefreshMainMenu();
+      if (main_menu != None) {
+	XtSetSensitive(XtNameToWidget(main_menu, "*use_xtest"), FALSE);
+	RefreshMainMenu();
+      }
     }
 #endif
 
